@@ -47,11 +47,11 @@ import (
 )
 
 const (
-	version    = "v1.24.0"
+	version    = "v1.25.0"
 	copyright  = "GoAWK " + version + " - Copyright (c) 2022 Ben Hoyt"
 	shortUsage = "usage: goawk [-F fs] [-v var=value] [-f progfile | 'prog'] [file ...]"
 	longUsage  = `Standard AWK arguments:
-  --csv             equivalent to "-i csv"
+  --csv             enable CSV input mode; equivalent to "-i csv"
   -F separator      field separator (default " ")
   -f progfile       load AWK source from progfile (multiple allowed)
   -v var=value      variable assignment (multiple allowed)
