@@ -17,11 +17,9 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"math"
 	"math/rand"
 	"os"
-	"os/exec"
 	"regexp"
 	"runtime"
 	"strconv"
@@ -81,9 +79,8 @@ type interp struct {
 	hadFiles      bool
 	input         io.Reader
 	inputBuffer   []byte
-	inputStreams  map[string]io.ReadCloser
-	outputStreams map[string]io.WriteCloser
-	commands      map[string]*exec.Cmd
+	inputStreams  map[string]inputStream
+	outputStreams map[string]outputStream
 	noExec        bool
 	noFileWrites  bool
 	noFileReads   bool
@@ -393,9 +390,8 @@ func newInterp(program *parser.Program) *interp {
 	p.outputRecordSep = "\n"
 	p.subscriptSep = "\x1c"
 
-	p.inputStreams = make(map[string]io.ReadCloser)
-	p.outputStreams = make(map[string]io.WriteCloser)
-	p.commands = make(map[string]*exec.Cmd)
+	p.inputStreams = make(map[string]inputStream)
+	p.outputStreams = make(map[string]outputStream)
 	p.scanners = make(map[string]*bufio.Scanner)
 
 	return p
@@ -602,7 +598,7 @@ func Exec(source, fieldSep string, input io.Reader, output io.Writer) error {
 	config := &Config{
 		Stdin:  input,
 		Output: output,
-		Error:  ioutil.Discard,
+		Error:  io.Discard,
 		Vars:   []string{"FS", fieldSep},
 	}
 	_, err = ExecProgram(prog, config)
